# 启动流程
> 原理跟Spring的是一样的，j-spring只是多了一套解析Bean源码的过程。


## 1.编译
> 这一步主要根据用户的启动参数，创建临时运行脚本。

```js
/**
  入口程序
  具体参数的应用请参考3.1参数详解
*/

const app = new SpringBoot({
	rootPath:__dirname,
  srcList:["./app"],
  tempJsName:".runtemp.js", //生成临时文件名称
  configPaths:['./resource/app-dev.yaml'], //资源目录名称
	moduleList:[
		ZcPlatformApiServiceScaner,
		ZcEmailServiceScaner
  ]
})


/**
  扫描srcList目录和npm包moduleList下的所有目录
  解析成BeanDefine，然后生成的构造器的映射
  并把信息写入.runtemp.js临时文件
  线上部署时，直接运行.runtemp.js文件即可。
*/
app.build();


/**
  创建子进程，然后运行.runtemp.js文件
  一般开发的时候使用app.start() 等同于 app.build().run();
*/
app.run();
```

## 2.运行
> node .runtemp.js  

```js
//part 1
const {SpringFactory} = require("j-spring");
/** generate lib */
const {SpringLog} = require("j-spring");
/** generate lib */

//part2
const { TransactionManager } = require('./app/Application.js')
const { Service } = require('./app/Application.js')
const { Application } = require('./app/Application.js')
const classReferences = {};
classReferences["TransactionManager"] = TransactionManager;
classReferences["Service"] = Service;
classReferences["Application"] = Application;

//part3
const args = JSON.parse(`{"pattern":"shell","rootPath":"D://gitRepository//j-spring","srcList":["D://gitRepository//j-spring//app"],"moduleList":[],"tempJsName":".runtemp.js","configPaths":["D://gitRepository//j-spring//resource//app-dev.yaml"],"inputArgs":[],"packageName":"./spring","logPackageName":"./spring","annotation":{"valueInject":"Value","appBoot":"SpringBoot","beanInject":"Autowired","springFactory":"SpringFactory","springResource":"SpringResource","proxy":"Proxy","logInject":"log"}}`);
 args.SpringLog = SpringLog;

 //part4
new SpringFactory(args,classReferences).boot();

```


- part1: 引用j-spring的工厂容器和启用的日志模块  
- part2: 将导出的Class添加到引用对象中
- part3: 配置参数
- part4: 到这里才是真正的启动（随后的原理基本上和Spring是一样的，比较简单）

## 3.注意

### 3.1 程序入口
SpringFactory装配工作完成后，会在最后调用带有@SpringBoot类的main方法，并传入启动参数。
```js

// 运行node .runtemp.js hello world

//@SpringBoot
class Application {
	async main(a,b){
    console.log(a,b) //hello world
	}
}
```

### 3.2 源码解析深度
这里的解析Js源码，只是简单的字面量解析并没有上升到AST的高度，所以无法自动注入父类的属性。因为j-spring的设计初衷就是轻量级应用，设计太复杂的东西会增加学习和维护成本。关键臣妾做不到啊。
