# BeanDefine
> 标识有注解的类将会解析得到BeanDefine,该类用于保存和操作类的元数据信息。


## 1.基础定义
```js
class BaseDefine {

  _constructor_;//类的构造器 SpringFactory初始化时注入


  /**
    Field 取字段名
    Method 取方法名
    BeanDefine 默认取类名；但使用@Bean(diyName)可以手动设定名称
  */

  name; //String 定义的名称（类名，方法名，字段名）

  annotation=[];//List<Annotation> 定义的注解对象集合

  /**
    判断是否存在某个注解
  */
  Boolean hasAnnotation(String annotationName);

  /**
    通过注解名称 返回指定注解 可能为Null
  */
  Annotation getAnnotation(String annotationName);
}
```

## 2.字段定义
```js
class Field extends BaseDefine {

}
```

## 3.方法定义
```js
class Method extends BaseDefine{
	params=[];//  type:List<String> 方法的参数集合
}
```

## 4.类的定义
```js
class BeanDefine extends BaseDefine {

  className;// String 类名

  fsPath;//String 文件路径

  fields=[]; //List<Field> 字段定义集合

  methods=[];//List<Method> 方法定义集合

  Field getField(String fieldName);//获取指定字段定义

  Method getMethod(String methodName)//后去指定方法定义
}
```

## 5.打印BeanDefine
```js

//@ClassAnnotation(msg=hello world,year=2022)
//@SpringBoot
class Application {

	//@FieldAnnotation
	name;

	//@MethodAnnotation1(c=!)
	//@MethodAnnotation2(a=xxx,b=ccc)
	say(a,b){

	}

	//@Autowired
	springFactory;//内置Bean


	async main(){
		//获取自身类的BeanDefine
		const define = this.springFactory.getBeanDefineByBean(this)
		//格式化打印
		console.log(JSON.stringify(define,null,2))
	}
}

/********** 输出结果  ************/

{
  "name": "Application",
  "annotation": [
    {
      "name": "SpringBoot",
      "param": {}
    },
    {
      "name": "ClassAnnotation",
      "param": {
        "msg": "hello world",
        "year": "2022"
      }
    }
  ],
  "className": "Application",
  "fsPath": "D:\\tempProject\\test-define\\app\\Application.js",
  "fields": [
    {
      "name": "name",
      "annotation": [
        {
          "name": "FieldAnnotation",
          "param": {}
        }
      ]
    },
    {
      "name": "springFactory",
      "annotation": [
        {
          "name": "Autowired",
          "param": {}
        }
      ]
    }
  ],
  "methods": [
    {
      "name": "say",
      "annotation": [
        {
          "name": "MethodAnnotation1",
          "param": {
            "c": "!"
          }
        },
        {
          "name": "MethodAnnotation2",
          "param": {
            "a": "xxx",
            "b": "ccc"
          }
        }
      ],
      "params": [
        "a",
        "b"
      ]
    }
  ]
}


```
