# 日志

## 参数配置
> app.yaml

```yaml
j-spring:
  log:
    state: on
    level: info
```

## 日志使用

```js
//@SpringBoot
class Application {

  //添加Log属性即可,容器会自动注入日志.并且添加类名信息
  log;

  async main(args){

    this.log.info('trace');
    this.log.info('debug');
    this.log.info('info');
    this.log.info('warn');
    this.log.info('error');

    const methodLog = this.log.method('main')
    methodLog.info('method log')

  }

}


/**
  输出:
  $ node index.js
  ********** Spring-Launch ************
  [当前输出级别]位置信息:输出内容
  [info]Application: trace
  [info]Application: debug
  [info]Application: info
  [info]Application: warn
  [info]Application: error
  [info]Application=>main: method log

  ********** Spring-Destroy ************
*/
```

## 扩展日志
> 日志是一个很复杂的模块,所以j-spring只做了简单的实现.但是留下了接口桥接其它成熟的框架.

### 1.扩展SpringLog
> 不继承也没事,实现所有的方法即可.

```js
const {SpringLog} = require('j-spring')

class OtherLog extends SpringLog {

  /**
    debugType String 当前输出级别
    className String 位置例如Application=>main ,代表Application类的main方法.
    param [Object] 传入的参数
  */
  extendLog(debugType,className,param){};

}

  /**
    这一步很重要,就是用来覆盖原有的日志类
  */
module.exports = {SpringLog:OtherLog}
```

### 2.启动配置
```js
const app = new SpringBoot({
	logPackageName:"your log packageName";//相对绝对都可.
})
```
